/* -----------------------------------------------------------------------
 EncodeSet.java
 Copyright (C) 2003  Nue

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
----------------------------------------------------------------------- */
/*
 * Created on 2003/03/06
 *
 * ե̾ EncodeSet.java
 */
package jp.co.jlight;

import java.nio.charset.*;
import java.io.*;
import java.util.*;
/**
 * 󥳡ǥ󥰤˻Ȥ饯åȤݻ
 */
public class EncodeSet {

	private ArrayList encs;
	
	// MS932к
//	private static final String defaultEncoding = System.getProperty("file.encoding");
	private String defaultEncoding = System.getProperty("file.encoding");
	
	private static final String SEPARATOR = System.getProperty("file.separator");
	private static ResourceBundle resources = ResourceBundle.getBundle("resources.jlighteditor");
	private File resFile;

	public EncodeSet() {
		String fileName = resources.getString("EncodeFileName");
		resFile = new File("resources" + SEPARATOR + fileName);
		encs = new ArrayList();
		createEncodeSet();
	}

	private void createEncodeSet() {
		int id = 0;
		String realName = defaultEncoding;
		String dispName = getDefaultEncoding() + " (Default)";
		Encode encode = new Encode(id, dispName, realName);
		encs.add(encode);

		Charset cs = null;
		
		// MS932к(㳰ȯj2sdk1.4.1)
//		Charset defcs = Charset.forName(defaultEncoding);
		if (defaultEncoding.equals("MS932")) {
			defaultEncoding = "SJIS";
		}
		Charset defcs = Charset.forName(defaultEncoding);



		FileReader fr = null;
		BufferedReader br = null;
		String line = "";
		try {
			fr = new FileReader(resFile.getPath());
			br = new BufferedReader(fr);
			int idx = 0;

			while ((line = br.readLine()) != null) {
				idx = line.indexOf('=');
				realName = line.substring(0, idx);

				try {
					cs = Charset.forName(realName);
					if (!cs.equals(defcs)) {
						dispName = dispName = line.substring(idx + 1);
						encs.add(new Encode(++id, dispName, realName));
					}
				} catch (IllegalCharsetNameException e) {
					System.err.println("IllegalCharsetNameException :" + e.getCharsetName());
				} catch (UnsupportedCharsetException e) {
					System.err.println("UnsupportedCharsetException :" + e.getCharsetName());
				}
			}

		} catch (FileNotFoundException e) {
			System.err.println("FileNotFoundException: " + 
								resFile.getAbsolutePath());
		} catch (IOException e) {
			System.err.println("IOException: readLine()");
		} finally {
			if (br != null) {
				try {
					br.close();
				} catch (IOException e) {
				}
			}
			if (fr != null) {
				try {
					fr.close();
				} catch (IOException e) {
				}
			}
		}
	}


	private Charset getDefaultCharset() {
		return Charset.forName(defaultEncoding);
	}

	public String getDefaultEncoding() {
		return defaultEncoding.toUpperCase();
	}

	public int getCount() {
		return encs.size();
	}

	public String getRealEncodeName(int id) {
		if (id < 0 | id > (getCount() - 1)) return "";
		Encode encode = (Encode)(encs.get(id));
		return encode.getRealEncoding();
	}

	public String getDispEncodeName(int id) {
		if (id < 0 | id > (getCount() - 1)) return "";
		Encode encode = (Encode)(encs.get(id));
		return encode.getDispEncoding();
	}



	class Encode {
		private int id;
		private String dispEncoding;
		private String realEncoding;

		public Encode(int id, String displayName, String realName) {
			this.id = id;
			this.dispEncoding = displayName;
			this.realEncoding = realName;
		}

		/**
		 * @return int
		 */
		public int getId() {
			return id;
		}
		/**
		 * @return String
		 */
		public String getDispEncoding() {
			return dispEncoding;
		}
		/**
		 * @return String
		 */
		public String getRealEncoding() {
			return realEncoding;
		}
	}

}
